import copy
from typing import Any, AsyncIterator, Iterator

# Note: Import directly from langchain_core is not stable and generate some errors
# from langchain_core.documents import Document
from langchain.schema import Document

from langchain_rag.document_transformers.runnable_document_transformer import (
    RunnableGeneratorDocumentTransformer,
)


class CopyDocumentTransformer(RunnableGeneratorDocumentTransformer):
    def lazy_transform_documents(
        self, documents: Iterator[Document], **kwargs: Any
    ) -> Iterator[Document]:
        yield from (copy.deepcopy(doc) for doc in documents)

    async def _alazy_transform_documents(  # type: ignore
        self, documents: AsyncIterator[Document], **kwargs: Any
    ) -> AsyncIterator[Document]:
        async for doc in documents:
            yield copy.deepcopy(doc)



from langchain_rag.document_transformers.document_transformers import (
    DocumentTransformers,
)
from langchain_rag.document_transformers.runnable_document_transformer import (
    RunnableGeneratorDocumentTransformer,
)

from .copy_transformer import CopyDocumentTransformer
from .generate_questions import GenerateQuestionsTransformer
from .sumarize_and_questions_transformer import (
    SummarizeAndQuestionsTransformer,
)
from .sumarize_transformer import SummarizeTransformer

__all__ = [
    "CopyDocumentTransformer",
    "GenerateQuestionsTransformer",
    "SummarizeAndQuestionsTransformer",
    "SummarizeTransformer",
    "DocumentTransformers",
    "RunnableGeneratorDocumentTransformer",
]
